import * as csvtojson from 'csvtojson';
import { BadRequestException, Injectable } from '@nestjs/common';
import { Equipo } from '../equipo/entity/equipo.entity';
import { Usuario } from '../usuario/entity/usuario.entity';
import { CarritoService } from '../carrito/carrito.service';
import { EquipoService } from '../equipo/equipo.service';
import { InstitucionService } from '../institucion/institucion.service';
import { InstitucionCarreraService } from '../institucion-carrera/institucion-carrera.service';
import { InstitucionProgramaService } from '../institucion-programa/institucion-programa.service';
import { InstitucionTipoCarritoService } from 'src/institucion-tipo-carrito/institucion-tipo-carrito.service';
import { ModuloService } from 'src/modulo/modulo.service';
import { TipoUsuarioService } from '../tipo-usuario/tipo-usuario.service';
import { UsuarioService } from '../usuario/usuario.service';
import { UploadFileCargaMasivaUsuarioDto } from './dto/upload-file-carga-masiva-usuario.dto';
import { UploadFileCargaMasivaEquipoDto } from './dto/upload-file-carga-masiva-equipo.dto';

@Injectable()
export class UploadFileService {
  constructor(
    private carritoService: CarritoService,
    private equipoService: EquipoService,
    private institucionService: InstitucionService,
    private institucionCarreraService: InstitucionCarreraService,
    private institucionProgramaService: InstitucionProgramaService,
    private institucionTipoCarritoService: InstitucionTipoCarritoService,
    private moduloService: ModuloService,
    private tipoUsuarioService: TipoUsuarioService,
    private usuarioService: UsuarioService,
  ) {}

  async createEquipos(file: Express.Multer.File, id_institucion: number) {
    const path = `${file.destination}/${file.filename}`;
    const institucion = await this.institucionService.findById(id_institucion);
    const errores: string[] = [];
    const mensajes: string[] = [];
    const equiposNuevos: Equipo[] = [];

    if (!file) throw new BadRequestException('No se mando ningún archivo.');
    return csvtojson()
      .fromFile(path)
      .then(async (equipos: [UploadFileCargaMasivaEquipoDto]) => {
        for (let i = 0; i < equipos.length; i++) {
          let error = this.errorBase(i);

          if (
            !equipos[i].carrito ||
            !equipos[i].entradas ||
            !equipos[i].equipo ||
            !equipos[i].marca ||
            !equipos[i].modelo ||
            !equipos[i].modulo ||
            !equipos[i].numero_inventario ||
            !equipos[i].numero_serie ||
            !equipos[i].tipo
          ) {
            if (!equipos[i].carrito) error += ' falta del campo carrito';
            if (error != this.errorBase(i)) error += ',';
            if (!equipos[i].entradas) error += ' falta del campo entradas';
            if (error != this.errorBase(i)) error += ',';
            if (!equipos[i].equipo) error += ' falta del campo equipo';
            if (error != this.errorBase(i)) error += ',';
            if (!equipos[i].marca) error += ' falta del campo marca';
            if (error != this.errorBase(i)) error += ',';
            if (!equipos[i].modelo) error += ' falta del campo modelo';
            if (error != this.errorBase(i)) error += ',';
            if (!equipos[i].modulo) error += ' falta del campo modulo';
            if (error != this.errorBase(i)) error += ',';
            if (!equipos[i].numero_inventario)
              error += ' falta del campo numero_inventario';
            if (error != this.errorBase(i)) error += ',';
            if (!equipos[i].numero_serie)
              error += ' falta del campo numero_serie';
            if (error != this.errorBase(i)) error += ',';
            if (!equipos[i].tipo) error += ' falta del campo tipo';
            errores.push(error + '.');
            continue;
          }

          const tipoCarrito =
            await this.institucionTipoCarritoService.findTipoCarritoByTipoCarrito(
              equipos[i].tipo,
              false,
            );
          const modulo = await this.moduloService.findModulo(
            institucion,
            equipos[i].modulo,
            false,
          );
          const carrito =
            tipoCarrito && modulo
              ? await this.carritoService.findCarrito(
                  modulo,
                  tipoCarrito,
                  equipos[i].modulo,
                  false,
                )
              : null;
          let equipo = carrito
            ? await this.equipoService.findEquipo(
                carrito,
                equipos[i].equipo,
                false,
              )
            : null;

          if (!tipoCarrito || !modulo || !carrito) {
            if (!tipoCarrito) error += ' no existe este tipo de carrito.';
            if (!modulo) error += ' no existe este módulo.';
            if (!carrito) error += ' no existe este carrito.';
            continue;
          }

          if (equipo) {
            error += ` ya existe un equipo de computo con el sobrenombre: ${equipo.equipo} en el carrito: ${equipo.carrito.carrito}`;
            continue;
          } else {
            equipo = await this.equipoService.findByNumeroInventario(
              institucion,
              equipos[i].numero_inventario,
              false,
            );
            if (equipo) {
              error += ' ya existe un equipo con ese número de inventario.';
              continue;
            }
            await this.equipoService
              .create(
                carrito,
                equipos[i].equipo,
                equipos[i].marca,
                equipos[i].modelo,
                equipos[i].numero_inventario,
                equipos[i].numero_serie,
              )
              .then((equipo) => equiposNuevos.push(equipo));
          }
        }
        return {
          response: {
            message: 'Se subió y cargó correctamente tu archivo csv.',
            mensajes,
            equiposNuevos,
            errores,
          },
          path,
        };
      });
  }

  async createUsuarios(file: Express.Multer.File, id_institucion: number) {
    const path = `${file.destination}/${file.filename}`;
    const institucion = await this.institucionService.findById(id_institucion);
    const errores: string[] = [];
    const mensajes: string[] = [];
    const usuariosNuevos: Usuario[] = [];

    if (!file) throw new BadRequestException('No se mando ningún archivo.');
    return csvtojson()
      .fromFile(path)
      .then(async (usuarios: [UploadFileCargaMasivaUsuarioDto]) => {
        for (let i = 0; i < usuarios.length; i++) {
          let error = this.errorBase(i);

          if (
            !usuarios[i].numero_cuenta ||
            !usuarios[i].tipo_usuario ||
            !usuarios[i].nombres ||
            !usuarios[i].apellido_m ||
            !usuarios[i].apellido_p
          ) {
            if (!usuarios[i].numero_cuenta)
              error += ' falta del campo numero_cuenta';
            if (error != this.errorBase(i)) error += ',';
            if (!usuarios[i].tipo_usuario)
              error += ' falta del campo tipo_usuario';
            if (error != this.errorBase(i)) error += ',';
            if (!usuarios[i].nombres) error += ' falta del campo nombres';
            if (error != this.errorBase(i)) error += ',';
            if (!usuarios[i].apellido_m) error += ' falta del campo apellido_m';
            if (error != this.errorBase(i)) error += ',';
            if (!usuarios[i].apellido_p) error += ' falta del campo apellido_p';
            errores.push(`${error}.`);
            continue;
          }

          const tipoUsuario = await this.tipoUsuarioService.findByTipoUsuario(
            usuarios[i].tipo_usuario,
            false,
          );
          const usuario = await this.usuarioService.findByUsuario(
            usuarios[i].numero_cuenta,
            false,
          );
          const carrera =
            tipoUsuario &&
            (tipoUsuario.id_tipo_usuario === 5 || usuarios[i].carrera)
              ? await this.institucionCarreraService.findCarreraByCarrera(
                  tipoUsuario.id_tipo_usuario === 5
                    ? 'PROFESOR'
                    : usuarios[i].carrera,
                  false,
                )
              : null;
          const institucionCarrera = carrera
            ? await this.institucionCarreraService.findByIdInstitucionIdCarrera(
                institucion,
                carrera,
              )
            : null;

          if (
            !tipoUsuario ||
            tipoUsuario.id_tipo_usuario < 5 ||
            !carrera ||
            (carrera && !institucionCarrera)
          ) {
            if (!tipoUsuario) error += ' no existe este tipo de usuario.';
            if (tipoUsuario.id_tipo_usuario < 5)
              ' no se puede asignar este tipo de usuario a este usuario.';
            if (!carrera) error += ' falta del campo carrera para el alumno.';
            if (carrera && !institucionCarrera)
              error += ' la carrera no pertenece a la institución.';
            errores.push(error);
            continue;
          }
          if (usuario) {
            if (
              usuario.tipoUsuario.id_tipo_usuario != tipoUsuario.id_tipo_usuario
            )
              usuario.tipoUsuario = tipoUsuario;
            if (!usuario.activo) usuario.activo = true;
            if (
              institucionCarrera &&
              usuario.institucionCarrera.id_institucion_carrera !=
                institucionCarrera.id_institucion_carrera
            )
              usuario.institucionCarrera = institucionCarrera;
            await this.usuarioService
              .update(usuario, false)
              .then(({ message }) => mensajes.push(message));
          } else
            await this.usuarioService
              .create(
                usuarios[i].numero_cuenta,
                `${usuarios[i].apellido_p.trim()} ${usuarios[
                  i
                ].apellido_m.trim()} ${usuarios[i].nombres.trim()}`,
                institucion,
                tipoUsuario,
                institucionCarrera,
              )
              .then((res) => {
                usuariosNuevos.push(res.usuario);
                mensajes.push(res.message);
              });
        }
        return {
          response: {
            message: 'Se subió y cargó correctamente tu archivo csv.',
            mensajes,
            usuariosNuevos,
            errores,
          },
          path,
        };
      });
  }

  downloadLogo(id_institucion: number) {}

  errorBase(index) {
    return `Se saltó la linea ${
      index + 2
    } por el siguiente/los siguientes motivos(s):`;
  }

  uploadLogo(file: Express.Multer.File, id_institucion: number) {}
}
